@@ -76,6 +76,12 @@ def find_grt_object(name, collection):
             return obj
     return None
 
+def fixQuoteIdentifier(name):
+    quotes = ['\'', '`', '"']
+    if name[0] in quotes:
+        name = name[1:-1]
+    return quoteIdentifier(name)
+
 ###############################################################################
 
 @ModuleInfo.export(grt.STRING, grt.STRING)
@@ -200,7 +206,8 @@ def getCatalogNames(connection):
 def getSchemaNames(connection, catalog_name):
     """Returns a list of schemata for the given connection object."""
 
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog_name))
+    query = 'USE %s' % quoteIdentifier(catalog_name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post70 = """SELECT TABLE_SCHEMA AS SCHEMANAME, max(TABLE_CATALOG) AS CATALOGNAME
     FROM INFORMATION_SCHEMA.TABLES
     WHERE TABLE_CATALOG = '%s'
@@ -218,12 +225,13 @@ GROUP BY TABLE_SCHEMA;""" % catalog_name
 
     query = query_pre70 if serverVersion.majorNumber <= 7 else query_post70
     #return [ row[1]+'.'+row[0] for row in execute_query(connection, query) ]
-    return [ row[0] for row in execute_query(connection, query) ]
+    return [ row[0] for row in execute_query(connection, query.decode(encoding='UTF-8',errors='strict')) ]
 
 
 @ModuleInfo.export(grt.LIST, grt.classes.db_mgmt_Connection, grt.STRING, grt.STRING)
 def getTableNames(connection, catalog_name, schema_name):
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog_name))
+    query = 'USE %s' % quoteIdentifier(catalog_name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT name
 FROM sys.tables
 WHERE schema_id = SCHEMA_ID('%s')""" % (schema_name)
@@ -235,12 +243,13 @@ WHERE TABLE_TYPE='BASE TABLE' AND TABLE_SCHEMA='%s'""" % schema_name
 
     query = query_pre_90 if serverVersion.majorNumber < 9 else query_post_90
 
-    return [ table_name[0] for table_name in execute_query(connection, query) ]
+    return [ table_name[0] for table_name in execute_query(connection, query.decode(encoding='UTF-8',errors='strict')) ]
 
 
 @ModuleInfo.export(grt.LIST, grt.classes.db_mgmt_Connection, grt.STRING, grt.STRING)
 def getViewNames(connection, catalog_name, schema_name):
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog_name))
+    query = 'USE %s' % quoteIdentifier(catalog_name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT name
 FROM sys.views
 WHERE SCHEMA_NAME(schema_id) = '%s';""" % schema_name
@@ -253,14 +262,15 @@ WHERE TABLE_CATALOG='%s' AND TABLE_SCHEMA='%s'""" % (catalog_name, schema_name)
         return []
     query = query_pre_90 if serverVersion.majorNumber < 9 else query_post_90
 
-    return [ views_info[0] for views_info in execute_query(connection, query) ]
+    return [ views_info[0] for views_info in execute_query(connection, query.decode(encoding='UTF-8',errors='strict')) ]
 
 
 @ModuleInfo.export(grt.LIST, grt.classes.db_mgmt_Connection, grt.STRING, grt.STRING)
 def getTriggerNames(connection, catalog_name, schema_name):
     # Query from http://msdn.microsoft.com/en-us/library/ms188746.aspx  (post 90)
     #            http://www.sharpdeveloper.net/content/archive/2007/06/26/search-trigger-text-sql-server-2005.aspx  (pre 90)
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog_name))
+    query = 'USE %s' % quoteIdentifier(catalog_name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT name
 FROM sys.triggers
 WHERE OBJECT_SCHEMA_NAME(object_id) = '%s'"""
@@ -274,14 +284,15 @@ GROUP BY OBJECT_NAME(id)"""
     if serverVersion.majorNumber <= 8:
         return []
     query = query_pre_90 if serverVersion.majorNumber <= 9 else query_post_90
-
-    return [ trigger_info[0] for trigger_info in execute_query(connection, query % schema_name) ]
+    query = query % schema_name
+    return [ trigger_info[0] for trigger_info in execute_query(connection, query.decode(encoding='UTF-8',errors='strict')) ]
 
 
 @ModuleInfo.export(grt.LIST, grt.classes.db_mgmt_Connection, grt.STRING, grt.STRING)
 def getProcedureNames(connection, catalog_name, schema_name):
     # Query from http://msdn.microsoft.com/en-us/library/ms345522.aspx#_FAQ9
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog_name))
+    query = 'USE %s' % quoteIdentifier(catalog_name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT name
 FROM sys.procedures
 WHERE schema_id = SCHEMA_ID('%s')"""
@@ -293,14 +304,15 @@ WHERE ROUTINE_SCHEMA='%s' AND ROUTINE_TYPE='PROCEDURE'"""
     if serverVersion.majorNumber <= 8:
         return []
     query = query_pre_90 if serverVersion.majorNumber < 9 else query_post_90
-
-    return [ proc_info[0] for proc_info in execute_query(connection, query % schema_name) ]
+    query = query % schema_name
+    return [ proc_info[0] for proc_info in execute_query(connection, query.decode(encoding='UTF-8',errors='strict')) ]
 
 
 @ModuleInfo.export(grt.LIST, grt.classes.db_mgmt_Connection, grt.STRING, grt.STRING)
 def getFunctionNames(connection, catalog_name, schema_name):
     # Query from http://msdn.microsoft.com/en-us/library/ms345522.aspx#_FAQ12
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog_name))
+    query = 'USE %s' % quoteIdentifier(catalog_name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT name
 FROM sys.objects
 WHERE type_desc LIKE '%FUNCTION%' AND schema_id = SCHEMA_ID(?)"""
@@ -313,7 +325,7 @@ WHERE ROUTINE_SCHEMA=? AND ROUTINE_TYPE='FUNCTION'"""
         return []
     query = query_pre_90 if serverVersion.majorNumber < 9 else query_post_90
 
-    return [ func_info[0] for func_info in execute_query(connection, query, schema_name) ]
+    return [ func_info[0] for func_info in execute_query(connection, query.decode(encoding='UTF-8',errors='strict'), schema_name.decode(encoding='UTF-8',errors='strict')) ]
 
 
 #########  Reverse Engineering functions #########
@@ -328,7 +340,7 @@ def reverseEngineer(connection, catalog_name, schemata_list, options):
     catalog.simpleDatatypes.remove_all()
     catalog.simpleDatatypes.extend(connection.driver.owner.simpleDatatypes)
 
-    collation_row = execute_query(connection, "SELECT DATABASEPROPERTYEX(?, 'Collation')", catalog_name).fetchone()
+    collation_row = execute_query(connection, "SELECT DATABASEPROPERTYEX(?, 'Collation')", catalog_name.decode(encoding='UTF-8',errors='strict')).fetchone()
     if collation_row:
         catalog.defaultCollationName = collation_row[0] or ''  # Avoid None in defaultCollationName
     
@@ -467,7 +479,8 @@ def reverseEngineer(connection, catalog_name, schemata_list, options):
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection, grt.classes.db_mssql_Catalog)
 def reverseEngineerUserDatatypes(connection, catalog):
     # TODO: Migrate user datatypes...
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog.name))
+    query = 'USE %s' % quoteIdentifier(catalog.name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query = """SELECT name, TYPE_NAME(system_type_id), max_length, precision, scale, is_nullable
 FROM sys.types
 WHERE is_user_defined = 1"""
@@ -539,7 +552,8 @@ def reverseEngineerTables(connection, schema):
 
     if is_first_pass:
         catalog = schema.owner
-        execute_query(connection, 'USE %s' % quoteIdentifier(catalog.name))
+        query = 'USE %s' % quoteIdentifier(catalog.name)
+        execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
         query_post_90 = """SELECT t.name, p.value
 FROM sys.tables t LEFT JOIN sys.extended_properties p ON p.major_id = t.object_id AND p.minor_id = 0 AND p.name = 'MS_Description' AND p.class_desc = 'OBJECT_OR_COLUMN'
 WHERE schema_id = SCHEMA_ID('%s')"""
@@ -551,9 +565,9 @@ WHERE schema_id = SCHEMA_ID('%s')"""
         serverVersion = connected_server_version(connection)
     
         query = query_pre_90 if serverVersion.majorNumber < 9 else query_post_90
-    
+        query = query % schema.name
         schema.tables.remove_all()
-        table_names = [(row[0], row[1]) for row in execute_query(connection, query % schema.name) ]
+        table_names = [(row[0], row[1]) for row in execute_query(connection, query.decode(encoding='UTF-8',errors='strict')) ]
         total = len(table_names) + 1e-10
         i = 0.0
         for table_name, table_comment in table_names:
@@ -587,7 +601,8 @@ WHERE schema_id = SCHEMA_ID('%s')"""
 def reverseEngineerTableColumns(connection, table):
     schema = table.owner
     catalog = schema.owner
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog.name))
+    query = 'USE %s' % quoteIdentifier(catalog.name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post90 = """SELECT sys.columns.name AS COLUMN_NAME,
     sys.columns.is_nullable AS IS_NULLABLE, sys.types.name AS DATA_TYPE, sys.columns.max_length AS CHARACTER_MAXIMUM_LENGTH,
     sys.columns.precision AS NUMERIC_PRECISION, sys.columns.scale AS NUMERIC_SCALE,
@@ -617,7 +632,7 @@ ORDER BY ORDINAL_POSITION"""
     serverVersion = connected_server_version(connection)
 
     query = query_pre90 if serverVersion.majorNumber < 9 else query_post90
-    rows = execute_query(connection, query, (schema.name, table.name) )
+    rows = execute_query(connection, query, (schema.name.decode(encoding='UTF-8',errors='strict'), table.name.decode(encoding='UTF-8',errors='strict')) )
 
     mssql_rdbms_instance = get_mssql_rdbms_instance()
     mssql_simple_datatypes_list = [ datatype.name for datatype in mssql_rdbms_instance.simpleDatatypes ]
@@ -694,14 +709,15 @@ def reverseEngineerTablePK(connection, table):
     schema = table.owner
     catalog = schema.owner
 
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog.name))
-    query = "exec sp_pkeys '%s', '%s'" % (table.name, schema.name)
+    query = 'USE %s' % quoteIdentifier(catalog.name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
+    query = "exec sp_pkeys '%s', '%s'" % (fixQuoteIdentifier(table.name), fixQuoteIdentifier(schema.name))
 
     if len(table.columns) == 0:
         grt.send_error('Migration: reverseEngineerTablePK', 'Reverse engineer of table %s.%s was attempted but the table has no columns attribute' % (schema.name, table.name) )
         return 1    # Table must have columns reverse engineered before we can rev eng its primary key(s)
 
-    pk_col_names = [ row[3] for row in execute_query(connection, query) ]
+    pk_col_names = [ row[3] for row in execute_query(connection, query.decode(encoding='UTF-8',errors='strict')) ]
 
     for pk_column in pk_col_names:
         column = find_grt_object(pk_column, table.columns)
@@ -723,7 +739,8 @@ def reverseEngineerTableIndices(connection, table):
 
     version = connected_server_version(connection)
 
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog.name))
+    query = 'USE %s' % quoteIdentifier(catalog.name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     if version.majorNumber <= 9:
         get_indices_query_pre90 = """SELECT u.name as TABLE_SCHEMA, 
                  o.name as TABLE_NAME, i.name AS INDEX_NAME, c.name AS COLUMN_NAME, 
@@ -739,8 +756,8 @@ def reverseEngineerTableIndices(connection, table):
                  i.indid > 0 AND i.indid < 255 AND o.type = 'U' AND 
                  (i.status & 64)=0 AND (i.status & 8388608)=0 AND 
                  (i.status & 2048)=0 AND  u.name='%s' AND o.name='%s'
-               ORDER BY i.name, k.keyno""" % (schema.name, table.name)
-        index_rows = execute_query(connection, get_indices_query_pre90).fetchall()
+               ORDER BY i.name, k.keyno""" % (fixQuoteIdentifier(schema.name), fixQuoteIdentifier(table.name))
+        index_rows = execute_query(connection, get_indices_query_pre90.decode(encoding='UTF-8',errors='strict')).fetchall()
         index = None
         for table_schema, table_name, index_name, column_name, ignore_dup_keys, is_unique, \
             ignore_dup_rows, is_clustered, is_primary_key, is_unique in index_rows:
@@ -771,18 +788,18 @@ def reverseEngineerTableIndices(connection, table):
         get_indices_query = """SELECT object_id, name, index_id, CAST (type_desc AS NVARCHAR) AS type_desc, is_unique,
     is_primary_key, is_disabled, has_filter, CAST (filter_definition AS NVARCHAR) AS filter_definition
     FROM sys.indexes
-    WHERE sys.indexes.object_id = OBJECT_ID('%s.%s')""" % (schema.name, table.name)
+    WHERE sys.indexes.object_id = OBJECT_ID('%s.%s')""" % (fixQuoteIdentifier(schema.name), fixQuoteIdentifier(table.name))
 
         get_index_columns_query = """SELECT c.name, ic.is_descending_key
     FROM sys.index_columns ic JOIN sys.columns c on (ic.column_id=c.column_id and ic.object_id=c.object_id)
     WHERE ic.object_id=%i and ic.index_id=%i"""
 
-        index_rows = execute_query(connection, get_indices_query).fetchall()
+        index_rows = execute_query(connection, get_indices_query.decode(encoding='UTF-8',errors='strict')).fetchall()
         for table_id, index_name, index_id, type_desc, is_unique, is_primary_key, is_disabled, has_filter, filter_definition in index_rows:
             if index_name is None:
                 # this is not a real index, but a HEAP thingmajig...
                 continue
-            if not is_disabled and not is_primary_key:    # Do not create the index if it is disabled or if it is part of the primary key
+            if not is_disabled: # Do not create the index if it is disabled
                 index = grt.classes.db_mssql_Index()
                 index.name = index_name
                 index.isPrimary = is_primary_key
@@ -813,7 +830,8 @@ def reverseEngineerTableFKs(connection, table):
     schema = table.owner
     catalog = schema.owner
 
-    execute_query(connection, 'USE %s' % quoteIdentifier(catalog.name))
+    query = 'USE %s' % quoteIdentifier(catalog.name)
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
 
     version = connected_server_version(connection)
     if version.majorNumber <= 9:
@@ -833,7 +851,7 @@ WHERE u.name=? AND t.name=? AND
  ref_tbl.id=sfk.rkeyid AND ref_tbl.uid=ref_u.uid AND 
  ref_c.id=ref_tbl.id AND ref_c.colid=sfk.rkey 
 ORDER BY sfk.constid, sfk.keyno"""
-        fk_rows = execute_query(connection, get_fks_query_pre9, (schema.name, table.name)).fetchall()
+        fk_rows = execute_query(connection, get_fks_query_pre9, (fixQuoteIdentifier(schema.name.decode(encoding='UTF-8',errors='strict')), fixQuoteIdentifier(table.name.decode(encoding='UTF-8',errors='strict')))).fetchall()
         foreign_key = None
         for fk_name, column_name, referenced_schema_name, referenced_table_name, referenced_column_name, update_rule, delete_rule in fk_rows:
             if foreign_key is None or foreign_key.name != fk_name:
@@ -875,7 +893,7 @@ ORDER BY sfk.constid, sfk.keyno"""
     else:
         get_fks_query = """SELECT object_id as fk_id, name, delete_referential_action_desc, update_referential_action_desc, is_disabled
     FROM sys.foreign_keys
-    WHERE parent_object_id=OBJECT_ID('%s.%s')""" % (schema.name, table.name)
+    WHERE parent_object_id=OBJECT_ID('%s.%s')""" % (fixQuoteIdentifier(schema.name), fixQuoteIdentifier(table.name))
 
         get_fk_mapping_query = """SELECT COL_NAME(fkc.parent_object_id, fkc.parent_column_id) as parent_column,
          OBJECT_SCHEMA_NAME(fkc.referenced_object_id) as referenced_schema,
@@ -889,7 +907,7 @@ ORDER BY sfk.constid, sfk.keyno"""
             grt.send_error('Migration: reverseEngineerTableFKs', 'Reverse engineer of table %s.%s was attempted but the table has no columns attribute' % (schema.name, table.name) )
             return 1    # Table must have columns reverse engineered before we can rev eng its foreign keys
 
-        fk_rows = execute_query(connection, get_fks_query).fetchall()
+        fk_rows = execute_query(connection, get_fks_query.decode(encoding='UTF-8',errors='strict')).fetchall()
         table.foreignKeys.remove_all()
         for fk_id, fk_name, delete_referential_action, update_referential_action, is_disabled in fk_rows:
             if not is_disabled:    # Do not create the foreign key if it is disabled
@@ -938,7 +956,8 @@ ORDER BY sfk.constid, sfk.keyno"""
 
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection, grt.classes.db_mssql_Schema)
 def reverseEngineerViews(connection, schema):
-    execute_query(connection, 'USE %s' % quoteIdentifier(schema.owner.name))  # catalog
+    query = 'USE %s' % quoteIdentifier(schema.owner.name)# catalog
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT COUNT(*) OVER () AS count, OBJECT_NAME(object_id) as name, definition
 FROM sys.sql_modules
 WHERE OBJECT_SCHEMA_NAME(object_id) = '%(schema)s'"""
@@ -951,8 +970,8 @@ WHERE TABLE_SCHEMA='%(schema)s'"""
     serverVersion = connected_server_version(connection)
 
     query = query_pre_90 if serverVersion.majorNumber <= 9 else query_post_90
-
-    cursor = execute_query(connection, query % {'schema':schema.name})
+    query = query % {'schema':schema.name}
+    cursor = execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     if cursor:
         schema.views.remove_all()
         step = 0.0
@@ -987,7 +1006,8 @@ def reverseEngineerView(connection, schema, view_name):
     """
     # The queries are taken from http://msdn.microsoft.com/en-us/library/ms345522.aspx#_FAQ35
 
-    execute_query(connection, 'USE %s' % quoteIdentifier(schema.owner.name))  # catalog
+    query = 'USE %s' % quoteIdentifier(schema.owner.name)# catalog
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT definition
 FROM sys.sql_modules
 WHERE object_id = OBJECT_ID('%s.%s')
@@ -999,8 +1019,8 @@ WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s'"""
     serverVersion = connected_server_version(connection)
 
     query = query_pre_90 if serverVersion.majorNumber < 9 else query_post_90
-
-    resultset = execute_query(connection, query % (schema.name, view_name) )
+    query = query % (schema.name, view_name)
+    resultset = execute_query(connection, query.decode(encoding='UTF-8',errors='strict') )
     if resultset:
         view = grt.classes.db_mssql_View()
         view.name = view_name or ""
@@ -1013,7 +1033,8 @@ WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s'"""
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection, grt.classes.db_mssql_Schema)
 def reverseEngineerProcedures(connection, schema):
     # http://msdn.microsoft.com/en-us/library/ms345443.aspx#TsqlProcedure
-    execute_query(connection, 'USE %s' % quoteIdentifier(schema.owner.name))  # catalog
+    query = 'USE %s' % quoteIdentifier(schema.owner.name)# catalog
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT COUNT(*) OVER () AS count, OBJECT_NAME(M.object_id) as name, definition
 FROM sys.sql_modules M JOIN sys.procedures P ON M.object_id=P.object_id
 WHERE OBJECT_SCHEMA_NAME(M.object_id) = '%(schema)s'"""
@@ -1026,8 +1047,8 @@ WHERE ROUTINE_SCHEMA='%(schema)s' AND ROUTINE_TYPE='PROCEDURE'"""
     serverVersion = connected_server_version(connection)
 
     query = query_pre_90 if serverVersion.majorNumber <= 9 else query_post_90
-
-    cursor = execute_query(connection, query % {'schema':schema.name})
+    query = query % {'schema':schema.name}
+    cursor = execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     step = 0.0
     if cursor:
         for idx, (proc_count, proc_name, proc_definition) in enumerate(cursor):
@@ -1046,7 +1067,8 @@ WHERE ROUTINE_SCHEMA='%(schema)s' AND ROUTINE_TYPE='PROCEDURE'"""
 
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection, grt.classes.db_mssql_Schema)
 def reverseEngineerFunctions(connection, schema):
-    execute_query(connection, 'USE %s' % quoteIdentifier(schema.owner.name))  # catalog
+    query = 'USE %s' % quoteIdentifier(schema.owner.name)# catalog
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT COUNT(*) OVER () AS count, OBJECT_NAME(M.object_id) as name, definition
 FROM sys.sql_modules M JOIN sys.objects O ON M.object_id=O.object_id
 WHERE type_desc LIKE '%%FUNCTION%%' AND schema_id = SCHEMA_ID('%(schema)s')"""
@@ -1059,8 +1081,8 @@ WHERE ROUTINE_SCHEMA='%(schema)s' AND ROUTINE_TYPE='FUNCTION'"""
     serverVersion = connected_server_version(connection)
 
     query = query_pre_90 if serverVersion.majorNumber < 9 else query_post_90
-
-    cursor = execute_query(connection, query % {'schema':schema.name})
+    query = query % {'schema':schema.name}
+    cursor = execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     step = 0.0
     if cursor:
         for idx, (func_count, func_name, func_definition) in enumerate(cursor):
@@ -1079,7 +1101,8 @@ WHERE ROUTINE_SCHEMA='%(schema)s' AND ROUTINE_TYPE='FUNCTION'"""
 
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection, grt.classes.db_mssql_Schema)
 def reverseEngineerTriggers(connection, schema):
-    execute_query(connection, 'USE %s' % quoteIdentifier(schema.owner.name))  # catalog
+    query = 'USE %s' % quoteIdentifier(schema.owner.name)# catalog
+    execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     query_post_90 = """SELECT st.object_id,
     st.name AS trigger_name,
     OBJECT_NAME(st.parent_id) AS table_name,
@@ -1107,8 +1130,8 @@ WHERE so.type = 'TR' AND su.name='%s'"""
     serverVersion = connected_server_version(connection)
 
     query = query_pre_90 if serverVersion.majorNumber <= 9 else query_post_90
-
-    cursor = execute_query(connection, query % schema.name)
+    query = query % schema.name
+    cursor = execute_query(connection, query.decode(encoding='UTF-8',errors='strict'))
     if cursor:
         for table in schema.tables:
             table.triggers.remove_all()
