@@ -1,37 +1,39 @@
-# Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 from __future__ import with_statement
 
 # import the mforms module for GUI stuff
 import mforms
-import grt
-import threading
 
 import sys, os, csv
 
-from mforms import newTreeNodeView
-from mforms import FileChooser
 import datetime
-import operator
 import json
+import base64
+from workbench.utils import Version
 
+from workbench.log import log_debug3, log_debug2, log_error, log_warning
 
-from workbench.log import log_debug3, log_debug2, log_error
+from wb_common import to_unicode
 
 last_location = ""
 
@@ -56,10 +58,10 @@ def handleContextMenu(name, sender, args):
     
     for s in selection:
         if s.type == 'db.Schema':
-            user_selection = {'schema': s.name, 'table': None}
+            user_selection = {'schema': to_unicode(s.name), 'table': None}
             break
         elif s.type == 'db.Table':
-            user_selection = {'table': s.name, 'schema': s.schemaName}
+            user_selection = {'table': to_unicode(s.name), 'schema': to_unicode(s.schemaName)}
             break
         else:
             return
@@ -89,6 +91,7 @@ class base_module:
         self._extension = None
         self._allow_remote = False
         self._editor = editor
+        self._targetVersion = Version.fromgrt(editor.serverVersion)
         self._local = True
         self._mapping = []
         self._new_table = False
@@ -103,16 +106,42 @@ class base_module:
         self._encoding = 'utf-8' #default encoding
         self._force_drop_table = False
         self._truncate_table = False
-        self._type_map = {'text':'is_string', 'int': 'is_number', 'double':'is_float', 'json': 'is_json'}
+        self._type_map = {'text':'is_string', 'bigint': 'is_bignumber', 'geometry': 'is_geometry', 'int': 'is_number', 'double':'is_float', 'json': 'is_json'}
         self.is_running = False
         self.progress_info = None
         self.item_count = 0
+        self._columnNames = {}
+
+    def reset_column_fix(self):
+        self._columnNames = {}
+
+    def fix_column_duplication(self, name):
+        # Just to be on the safe side if the column name is not a plain string so python is happy
+        bname = base64.b64encode(name)
+        if bname in self._columnNames:
+            name = "%s_[%d]" % (name, self._columnNames[bname])
+            self._columnNames[bname] += 1
+        else:
+            self._columnNames[bname] = 0
+        return name
+
+    def fix_column_name(self, name):
+        if name is None:
+            name = ""
+        name = name.strip()
+        if len(name) == 0:
+            name = "MyUnknownColumn"
+
+        return self.fix_column_duplication(name)
 
     def guess_type(self, vals):
         def is_json(v):
             return True if type(v) in [dict, list] else False
         
         def is_float(v):
+            if hasattr(v, 'encode'):
+                v = v.encode('utf8')
+
             v = str(v)
             try:
                 if "%s" % float(v) == v:
@@ -122,6 +151,9 @@ class base_module:
                 return False
         
         def is_int(v):
+            if hasattr(v, 'encode'):
+                v = v.encode('utf8')
+
             v = str(v)
             try:
                 if "%s" % int(v) == v:
@@ -136,7 +168,12 @@ class base_module:
                 return "json" # If JSON then we can return immediately. 
             if is_int(v):
                 if not cur_type:
-                    cur_type = "int"
+                    #We know that it's int, but let's see if it's maybe big int
+                    val = int(v)
+                    if val > 2147483647 or val < -2147483647:
+                        cur_type = "bigint"
+                    else:
+                        cur_type = "int"
                 continue
     
             if is_float(v):
@@ -216,7 +253,9 @@ class base_module:
             self._columns.append({'name': c.name, 'type': c.columnType, 
                    'is_string': any(x in c.columnType for x in ['char', 'text', 'set', 'enum']),
                    'is_json': any(x in c.columnType for x in ['json']), 
-                   'is_number': any(x in c.columnType for x in ['int', 'integer']), 
+                   'is_number': any(x in c.columnType for x in ['int', 'integer']),
+                   'is_bignumber':  any(x in c.columnType for x in ['bigint']),
+                   'is_geometry':  any(x in c.columnType for x in ['geometry','geometrycollection', 'linestring', 'multilinestring', 'multipoint', 'multipolygon', 'point' , 'polygon']),
                    'is_date_or_time': any(x in c.columnType for x in ['timestamp', 'time', 'datetime', 'date']), 
                    'is_bin': any(x in c.columnType for x in ['geo', 'blob', 'binary']),
                    'is_float': any(x in c.columnType for x in ['decimal', 'float', 'double', 'real']),
@@ -224,8 +263,7 @@ class base_module:
     
     def prepare_new_table(self):
         try:
-            
-            self._editor.executeManagementCommand(""" CREATE TABLE %s (%s)""" % (self._table_w_prefix, ", ".join(["`%s` %s" % (col['name'], col["type"]) for col in self._mapping])), 1)
+            self._editor.executeManagementCommand(""" CREATE TABLE %s (%s)""" % (self._table_w_prefix, ", ".join(["`%s` %s" % (col['name'], col["type"]) for col in self._mapping if col['active'] is True])), 1)
             self.update_progress(0.0, "Prepared new table")
             # wee need to setup dest_col for each row, as the mapping is empty if we're creating new table
             for col in self._mapping:
@@ -287,7 +325,7 @@ class UniReader:
     def next(self):
         row = self.csvreader.next()
         return [unicode(s, "utf-8") for s in row]
-    
+        
     @property
     def line_num(self):
         return self.csvreader.line_num
@@ -303,9 +341,10 @@ class csv_module(base_module):
         base_module.__init__(self, editor, is_import)
         self.name = "csv"
         self.title = self.name
-        self.options = {'filedseparator': {'description':'Field Separator', 'type':'select', 'opts':{'TAB':'\t',';':';', ':':':', ',':','}, 'value':';', 'entry': None},
+        self.options = {'filedseparator': {'description':'Field Separator', 'type':'select', 'opts':{'TAB':'\t','|':'|',';':';', ':':':', ',':','}, 'value':';', 'entry': None},
                 'lineseparator': {'description':'Line Separator', 'type':'select','opts':{"CR":'\r', "CR LF":'\r\n', "LF":'\n'}, 'value':'\n', 'entry': None}, 
-                'encolsestring': {'description':'Enclose Strings in', 'type':'text', 'value':'"', 'entry': None}};
+                'encolsestring': {'description':'Enclose Strings in', 'type':'text', 'value':'"', 'entry': None},
+                'nullwordaskeyword': {'description':'null and NULL word as SQL keyword', 'type':'select', 'opts':{'YES':'y', 'NO':'n'}, 'value':'y', 'entry': None}};
         
         self._extension = ["Comma Separated Values (*.csv)|*.csv", "csv"]
         self._allow_remote = True 
@@ -317,7 +356,7 @@ class csv_module(base_module):
                 limit = "LIMIT %d" % int(self._limit)
                 if self._offset:
                     limit = "LIMIT %d,%d" % (int(self._offset), int(self._limit))
-            return """SELECT %s FROM %s %s""" % (",".join([value['name'] for value in self._columns]), self._table_w_prefix, limit)
+            return """SELECT %s FROM %s %s""" % (",".join(["`%s`" % value['name'] for value in self._columns]), self._table_w_prefix, limit)
         else:
             limit = ""
             if self._limit:
@@ -331,7 +370,7 @@ class csv_module(base_module):
             return """SELECT %s FROM %s INTO OUTFILE '%s' 
                         FIELDS TERMINATED BY '%s' 
                         ENCLOSED BY '%s' 
-                        LINES TERMINATED BY %s %s""" % (",".join([value['name'] for value in self._columns]), self._table_w_prefix, fpath,
+                        LINES TERMINATED BY %s %s""" % (",".join(["`%s`" % value['name'] for value in self._columns]), self._table_w_prefix, fpath,
                                                        self.options['filedseparator']['value'], self.options['encolsestring']['value'], repr(self.options['lineseparator']['value']), limit)
 
     def start_export(self):
@@ -374,10 +413,12 @@ class csv_module(base_module):
                         self.update_progress(round(self._current_row / self._max_rows, 2), "Data export")
                         row = []
                         for col in self._columns:
-                            if col['is_number']:
+                            if col['is_number'] or col['is_bignumber']:
                                 row.append(rset.intFieldValueByName(col['name']))
                             elif col['is_float']:
                                 row.append(rset.floatFieldValueByName(col['name']))
+                            elif col['is_geometry']:
+                                row.append(rset.geoStringFieldValueByName(col['name']))
                             else:
                                 row.append(rset.stringFieldValueByName(col['name']))
                         output.writerow(row)
@@ -410,6 +451,8 @@ class csv_module(base_module):
             col_order = dict([(i['dest_col'], i['col_no']) for i in self._mapping if i['active']])
             col_type = dict([(i['dest_col'], i['type']) for i in self._mapping if i['active']])
 
+            is_server_5_7 = self._targetVersion.is_supported_mysql_version_at_least(Version.fromstr("5.7.5"))
+            
             self._editor.executeManagementCommand(query, 1)
             try:
                 is_header = self.has_header
@@ -430,7 +473,7 @@ class csv_module(base_module):
                         continue
 
                     
-                    self.update_progress(round(self._current_row / self._max_rows, 2), "Data import")
+                    
 
                     for i, col in enumerate(col_order):
                         if col_order[col] >= len(row):
@@ -439,20 +482,36 @@ class csv_module(base_module):
                             break
                         val = row[col_order[col]]
                         col_name = col_order[col]
-                        if col_type[col] == 'double':
-                            val = row[col_name].replace(self._decimal_separator, '.')
-                        elif col_type[col] == 'datetime':
-                            val = datetime.datetime.strptime(row[col_name], self._date_format).strftime("%Y-%m-%d %H:%M:%S")
-                            
-                        if hasattr(val, "replace"):
-                            val = val.replace("\\", "\\\\").replace('"', '\\"')
-                        self._editor.executeManagementCommand("""SET @a%d = "%s" """ % (i, val), 0)
+
+                        if col_type[col] == "geometry":
+                            if is_server_5_7:
+                                val = """ST_GeomFromText("%s")""" % row[col_name]
+                            else:
+                                val = """GeomFromText("%s")""" % row[col_name]
+
+                            self._editor.executeManagementCommand("""SET @a%d = %s """ % (i, val), 0)
+                        else:
+                            if col_type[col] == 'double':
+                                val = row[col_name].replace(self._decimal_separator, '.')
+                            elif col_type[col] == 'datetime':
+                                val = datetime.datetime.strptime(row[col_name], self._date_format).strftime("%Y-%m-%d %H:%M:%S")
+                            if hasattr(val, "replace"):
+                                val = val.replace("\\", "\\\\").replace("'", "\\'")
+
+                            if self.options['nullwordaskeyword']['value'] == "y" and val.upper() == "NULL":
+                                self._editor.executeManagementCommand("""SET @a%d = NULL """ % (i), 0)
+                            else:
+                                self._editor.executeManagementCommand("""SET @a%d = '%s' """ % (i, val), 0)
                     else:
                         try:
                             self._editor.executeManagementCommand("EXECUTE stmt USING %s" % ", ".join(['@a%d' % i for i, col in enumerate(col_order)]), 0)
                             self.item_count = self.item_count + 1
+                            self.update_progress(round(self._current_row / self._max_rows, 2), "Data import")
                         except Exception, e:
                             log_error("Row import failed with error: %s" % e)
+                            self.update_progress(round(self._current_row / self._max_rows, 2), "Row import failed with error: %s" % e)
+                            result = False
+
                 self.update_progress(1.0, "Import finished")
             except Exception, e:
                 import traceback
@@ -463,15 +522,22 @@ class csv_module(base_module):
         return result
 
     def analyze_file(self):
+        self.reset_column_fix()
         with open(self._filepath, 'rb') as csvfile:
             if self.dialect is None:
                 csvsample = []
                 for i in range(0,2): #read two lines as a sample
-                    csvsample.append(csvfile.readline())
-                
+                    line = csvfile.readline()
+                    if len(line) > 0:
+                        csvsample.append(line)
+
+                csvsample_len = len(csvsample)
                 csvsample = "".join(csvsample)
                 self.dialect = csv.Sniffer().sniff(csvsample)
                 self.has_header = csv.Sniffer().has_header(csvsample)
+                if self.has_header and csvsample_len == 1:
+                    self.has_header = False
+
                 csvfile.seek(0)
                 self.options['filedseparator']['value'] = self.dialect.delimiter 
                 self.options['lineseparator']['value'] = self.dialect.lineterminator 
@@ -480,23 +546,26 @@ class csv_module(base_module):
                 self.dialect.delimiter = self.options['filedseparator']['value']
                 self.dialect.lineterminator = self.options['lineseparator']['value']
                 self.dialect.quotechar = self.options['encolsestring']['value']
+
                 csvfile.seek(0)
                 
             try:
                 reader = UniReader(csvfile, self.dialect, encoding=self._encoding)
                 self._columns = []
-                row = None
+                row_line = None
                 try:
-                    row = reader.next()
+                    row_line = reader.next()
                 except StopIteration, e:
                     pass
                 
-                if row:
-                    for col_value in row:
-                        self._columns.append({'name': col_value, 'type': 'text', 'is_string': True, 'is_number': False, 'is_date_or_time': False, 'is_bin': False, 'is_float':False, 'is_json':False,'value': []})
-
-                    for i, row in enumerate(reader): #we will read only first few rows
-                        if i < 5:
+                
+                if row_line:
+                    for col_value in row_line:
+                        self._columns.append({'name': self.fix_column_name(col_value) , 'type': 'text', 'is_string': True, 'is_geometry': False, 'is_bignumber': False, 'is_number': False, 'is_date_or_time': False, 'is_bin': False, 'is_float':False, 'is_json':False,'value': []})
+                        
+                    ii = -1
+                    for ii, row in enumerate(reader): #we will read only first few rows
+                        if ii < 5:
                             for j, col_value in enumerate(row):
                                 try:
                                     json_value = json.loads(col_value)
@@ -507,9 +576,27 @@ class csv_module(base_module):
                                     self._columns[j]['value'].append(col_value)
                         else:
                             break
+
+                    # We hit here an edge case, enumerate didn't run but row_line contains something, 
+                    # we will assume this means that there's just one line
+                    if ii == -1 and len(row_line) > 0:
+                        ii = 1
+
+                    if not self.has_header and ii == 1: # This means there were one line which was consumed as a header we need to copy it to use as values
+                        log_warning("File: %s, probably has got only one line, using it as a header and data\n" % self._filepath)
+                        for j, col_value in enumerate(row_line):
+                            try:
+                                json_value = json.loads(col_value)
+                                self._columns[j]['is_string'] = False
+                                self._columns[j]['is_json'] = True
+                                self._columns[j]['value'].append(json_value)
+                            except Exception as e:
+                                self._columns[j]['value'].append(col_value)
+
                     for col in self._columns:
                         # Means the file is missing some data or is mallformed
                         if len(col['value']) == 0:
+                            log_error("Error analyzing file, we have no values.")
                             return False
 
                         gtype = self.guess_type(col['value'])
@@ -523,7 +610,6 @@ class csv_module(base_module):
                                         col[attrib] = True
                                     else:
                                         col[attrib] = False
-
             except (UnicodeError, UnicodeDecodeError), e:
                 import traceback
                 log_error("Error analyzing file, probably encoding issue: %s\n Traceback is: %s" % (e, traceback.format_exc()))
@@ -547,7 +633,7 @@ class json_module(base_module):
             limit = "LIMIT %d" % int(self._limit)
             if self._offset:
                 limit = "LIMIT %d,%d" % (int(self._offset), int(self._limit))
-        return """SELECT %s FROM %s %s""" % (",".join([value['name'] for value in self._columns]), self._table_w_prefix, limit)                
+        return u"""SELECT %s FROM %s %s""" % (",".join(["`%s`" % value['name'] for value in self._columns]), self._table_w_prefix, limit)                
     
     def start_export(self):
         if self._user_query:
@@ -584,17 +670,20 @@ class json_module(base_module):
                     self._current_row = rset.currentRow + 1
                     row = []
                     for col in self._columns:
-                        if col['is_number']:
+                        if col['is_number'] or col['is_bignumber']:
                             row.append("\"%s\":%s" % (col['name'], json.dumps(rset.intFieldValueByName(col['name']))))
                         elif col['is_float']:
                             row.append("\"%s\":%s" % (col['name'], json.dumps(rset.floatFieldValueByName(col['name']))))
+                        elif col['is_geometry']:
+                            row.append("\"%s\":%s" % (col['name'], rset.geoJsonFieldValueByName(col['name'])))
                         else:
                             if col['type'] == "json":
-                                row.append("\"%s\":%s" % (col['name'], rset.stringFieldValueByName(col['name'])))
+                                row.append(u"\"%s\":%s" % (col['name'], to_unicode(rset.stringFieldValueByName(col['name']))))
                             else:
-                                row.append("\"%s\":%s" % (col['name'], json.dumps(rset.stringFieldValueByName(col['name']))))
+                                row.append("\"%s\":%s" % (col['name'], json.dumps(to_unicode(rset.stringFieldValueByName(col['name'])))))
                     ok = rset.nextRow()
-                    jsonfile.write("{%s}%s" % (', '.join(row), ",\n " if ok else ""))
+                    line = u"{%s}%s" % (', '.join(row), ",\n " if ok else "")
+                    jsonfile.write(line.encode('utf-8'))
                     jsonfile.flush()
                 jsonfile.write(']')
 
@@ -616,9 +705,10 @@ class json_module(base_module):
         with open(self._filepath, 'rb') as jsonfile:
             data = json.load(jsonfile)
             dest_col_order = list(set([i['dest_col'] for i in self._mapping if i['active']]))
-            query = """PREPARE stmt FROM 'INSERT INTO %s (%s) VALUES(%s)'""" % (self._table_w_prefix, ",".join(dest_col_order), ",".join(["?" for i in dest_col_order]))
+            query = """PREPARE stmt FROM 'INSERT INTO %s (%s) VALUES(%s)'""" % (self._table_w_prefix, ",".join(["`%s`" % col for col in dest_col_order]), ",".join(["?" for i in dest_col_order]))
             col_order = dict([(i['dest_col'], i['name']) for i in self._mapping if i['active']])
             col_type = dict([(i['name'], i['type']) for i in self._mapping if i['active']])
+            
             self._editor.executeManagementCommand(query, 1)
             try:
                 self._max_rows = len(data)
@@ -636,23 +726,27 @@ class json_module(base_module):
 
                         val = row[col_order[col]]
                         col_name = col_order[col]
-                        if col_type[col_name] != "json" and hasattr(val, "replace"):
-                            val = val.replace("\\", "\\\\").replace('"', '\\"')
-                            
-                        if col_type[col_name] == 'double':
-                            val = val(str).replace(self._decimal_separator, '.')
-                        elif col_type[col_name] == 'datetime':
-                            val = datetime.datetime.strptime(val, self._date_format).strftime("%Y-%m-%d %H:%M:%S")
-                        elif col_type[col_name] == "json":
-                            val = json.dumps(val).replace("\\", "\\\\").replace("'", "\\'")
 
-                        if col_type[col_name] == "int":
-                            self._editor.executeManagementCommand("""SET @a%d = %d """ % (i, val), 0)
-                        elif col_type[col_name] == "json":
-                            self._editor.executeManagementCommand("""SET @a%d = '%s' """ % (i, val), 0)
+                        if col_type[col] == "geometry":
+                            val = """ ST_GeomFromGeoJSON('%s')""" % json.dumps(val).replace("\\", "\\\\").replace("'", "\\'")
+                                 
+                            self._editor.executeManagementCommand("""SET @a%d = %s """ % (i, val), 0)
                         else:
-                            self._editor.executeManagementCommand("""SET @a%d = "%s" """ % (i, val), 0)
-                        
+                            if col_type[col_name] != "json" and hasattr(val, "replace"):
+                                val = val.replace("\\", "\\\\").replace("'", "\\'")
+                                
+                            if col_type[col_name] == 'double':
+                                val = val(str).replace(self._decimal_separator, '.')
+                            elif col_type[col_name] == 'datetime':
+                                val = datetime.datetime.strptime(val, self._date_format).strftime("%Y-%m-%d %H:%M:%S")
+                            elif col_type[col_name] == "json":
+                                val = json.dumps(val).replace("\\", "\\\\").replace("'", "\\'")                                
+    
+                            if col_type[col_name] == "int":
+                                self._editor.executeManagementCommand("""SET @a%d = %d """ % (i, int(val)), 0)
+                            else:
+                                self._editor.executeManagementCommand("""SET @a%d = '%s' """ % (i, val), 0)
+
                     else:
                         try:
                             self._editor.executeManagementCommand("EXECUTE stmt USING %s" % ", ".join(['@a%d' % i for i, col in enumerate(col_order)]), 0)
@@ -677,8 +771,8 @@ class json_module(base_module):
             datachunk = []
             rowcount = 0 
             while True:
-                if f.tell() >= 4096:
-                    log_error("Json file contains data that's in unknown structure: %s" % (self._filepath))
+                if f.tell() >= 20480:
+                    log_error("JSON file contains data that's in unknown structure: %s" % (self._filepath))
                     return False
                 c = f.read(1)
                 if c == "":
@@ -704,17 +798,21 @@ class json_module(base_module):
             try:
                 data = json.loads("".join(datachunk))
             except Exception, e:
-                log_error("Unable to parse json file: %s,%s " % (self._filepath, e))
+                log_error("Unable to parse JSON file: %s,%s " % (self._filepath, e))
                 self._last_analyze = False
                 return False
         if len(data) == 0:
-            log_error("Json file contains no data, or data is inalivd: %s" % (self._filepath))
+            log_error("JSON file contains no data, or data is invalid: %s" % (self._filepath))
             self._last_analyze = False
             return False
         
         self._columns = []
+        
+        if type(data) == dict: # We need to have list so if it's dict after that we will be able to handle it.
+            data = [data]
+        
         for elem in data[0]:
-            self._columns.append({'name': elem, 'type': 'text', 'is_string': True, 'is_number': False, 'is_date_or_time': False, 'is_bin': False, 'is_float':False, 'is_json':False, 'value': []})
+            self._columns.append({'name': elem, 'type': 'text', 'is_string': True, 'is_geometry': False, 'is_bignumber': False, 'is_number': False, 'is_date_or_time': False, 'is_bin': False, 'is_float':False, 'is_json':False, 'value': []})
 
         for row in data:
             for i, elem in enumerate(row):
@@ -735,7 +833,6 @@ class json_module(base_module):
                             col[attrib] = True
                         else:
                             col[attrib] = False
-        
         self._last_analyze = True
         return True
         
